@@ -35,6 +35,7 @@ import jakarta.faces.component.visit.VisitHint;
 import jakarta.faces.component.visit.VisitResult;
 import jakarta.faces.context.FacesContext;
 import jakarta.faces.event.AbortProcessingException;
+import jakarta.faces.event.ActionListener;
 import jakarta.faces.event.ComponentSystemEvent;
 import jakarta.faces.event.ComponentSystemEventListener;
 import jakarta.faces.event.FacesEvent;
@@ -45,6 +46,7 @@ import org.apache.myfaces.tobago.component.Attributes;
 import org.apache.myfaces.tobago.component.Visual;
 import org.apache.myfaces.tobago.event.PageActionEvent;
 import org.apache.myfaces.tobago.event.SheetAction;
+import org.apache.myfaces.tobago.event.SheetRowSelectionChangeEvent;
 import org.apache.myfaces.tobago.event.SheetStateChangeEvent;
 import org.apache.myfaces.tobago.event.SheetStateChangeListener;
 import org.apache.myfaces.tobago.event.SheetStateChangeSource;
@@ -62,6 +64,7 @@ import org.apache.myfaces.tobago.model.ScrollPosition;
 import org.apache.myfaces.tobago.model.SelectedState;
 import org.apache.myfaces.tobago.model.SheetState;
 import org.apache.myfaces.tobago.util.ComponentUtils;
+import org.apache.myfaces.tobago.util.SearchOnce;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -98,6 +101,8 @@ public abstract class AbstractUISheet extends AbstractUIData
   private transient int lazyLastRow;
 
   private transient Grid headerGrid;
+  private final transient SearchOnce abstractUIColumnSelectorSearch = new SearchOnce();
+  private final transient SearchOnce abstractUIColumnPanelSearch = new SearchOnce();
 
   @Override
   public void encodeAll(FacesContext facesContext) throws IOException {
@@ -518,6 +523,15 @@ public abstract class AbstractUISheet extends AbstractUIData
       facesEvent.setPhaseId(PhaseId.INVOKE_APPLICATION);
       parent.queueEvent(facesEvent);
     } else {
+      if (facesEvent.getComponent() == this && facesEvent instanceof SheetRowSelectionChangeEvent) {
+        if (getColumnSelector() != null && getColumnSelector().isImmediate()) {
+          facesEvent.setPhaseId(PhaseId.APPLY_REQUEST_VALUES);
+        } else {
+          /* This should not happen, because SheetRowSelectionChangeEvent is only queued if
+          columnSelector immediate=true. See SheetRenderer#decodeInternal(). */
+          facesEvent.setPhaseId(PhaseId.INVOKE_APPLICATION);
+        }
+      }
       super.queueEvent(facesEvent);
     }
   }
@@ -539,6 +553,13 @@ public abstract class AbstractUISheet extends AbstractUIData
     } else if (facesEvent instanceof SortActionEvent) {
       getSheetState(getFacesContext()).updateSortState(((SortActionEvent) facesEvent).getColumn().getId());
       sort(getFacesContext(), (SortActionEvent) facesEvent);
+    } else if (facesEvent instanceof SheetRowSelectionChangeEvent sheetRowSelectionChangeEvent) {
+      if (getClientId().equals(sheetRowSelectionChangeEvent.getSourceId())) {
+        final ActionListener defaultActionListener = getFacesContext().getApplication().getActionListener();
+        if (defaultActionListener != null) {
+          defaultActionListener.processAction(sheetRowSelectionChangeEvent);
+        }
+      }
     }
   }
 
@@ -904,6 +925,14 @@ public abstract class AbstractUISheet extends AbstractUIData
     }
   }
 
+  public AbstractUIColumnSelector getColumnSelector() {
+    return abstractUIColumnSelectorSearch.findChild(this, AbstractUIColumnSelector.class);
+  }
+
+  public AbstractUIColumnPanel getColumnPanel() {
+    return abstractUIColumnPanelSearch.findChild(this, AbstractUIColumnPanel.class);
+  }
+
   @Override
   public boolean isRendersRowContainer() {
     return true;
